@@ -943,6 +943,40 @@ def test_can_create_app_packager_with_no_autogen(tmpdir):
     assert 'sam.json' in contents
 
 
+def test_can_create_app_packager_with_yaml_extention(tmpdir):
+    appdir = _create_app_structure(tmpdir)
+
+    outdir = tmpdir.mkdir('outdir')
+    default_params = {'autogen_policy': True}
+    extras_file = tmpdir.join('extras.yaml')
+    extras_file.write("foo: bar")
+    config = Config.create(project_dir=str(appdir),
+                           chalice_app=sample_app(),
+                           **default_params)
+    p = package.create_app_packager(config, merge_template=str(extras_file))
+
+    p.package_app(config, str(outdir), 'dev')
+    contents = os.listdir(str(outdir))
+    assert 'deployment.zip' in contents
+    assert 'sam.yaml' in contents
+
+
+def test_can_specify_yaml_output(tmpdir):
+    appdir = _create_app_structure(tmpdir)
+
+    outdir = tmpdir.mkdir('outdir')
+    default_params = {'autogen_policy': True}
+    config = Config.create(project_dir=str(appdir),
+                           chalice_app=sample_app(),
+                           **default_params)
+    p = package.create_app_packager(config, template_format='yaml')
+
+    p.package_app(config, str(outdir), 'dev')
+    contents = os.listdir(str(outdir))
+    assert 'deployment.zip' in contents
+    assert 'sam.yaml' in contents
+
+
 def test_will_create_outdir_if_needed(tmpdir):
     appdir = _create_app_structure(tmpdir)
     outdir = str(appdir.join('outdir'))
