@@ -64,6 +64,8 @@ class TestApplicationGraphBuilder(object):
                       api_gateway_endpoint_type=None,
                       api_gateway_endpoint_vpce=None,
                       api_gateway_policy_file=None,
+                      api_gateway_custom_domain=None,
+                      websocket_api_custom_domain=None,
                       project_dir='.'):
         kwargs = {
             'chalice_app': app,
@@ -72,7 +74,9 @@ class TestApplicationGraphBuilder(object):
             'api_gateway_stage': api_gateway_stage,
             'api_gateway_policy_file': api_gateway_policy_file,
             'api_gateway_endpoint_type': api_gateway_endpoint_type,
-            'api_gateway_endpoint_vpce': api_gateway_endpoint_vpce
+            'api_gateway_endpoint_vpce': api_gateway_endpoint_vpce,
+            'api_gateway_custom_domain': api_gateway_custom_domain,
+            'websocket_api_custom_domain': websocket_api_custom_domain,
         }
         if iam_role_arn is not None:
             # We want to use an existing role.
@@ -155,9 +159,35 @@ class TestApplicationGraphBuilder(object):
             reserved_concurrency=None,
         )
 
-    def test_can_build_lambda_function_app_with_vpc_config(
-            self, sample_app_lambda_only):
+    def test_can_build_app_with_domain_name(self, sample_app):
+        domain_name = {
+            'domain_name': 'example.com',
+            'tls_version': 'TLS_1_0',
+            'certificate_arn': 'certificate_arn',
+            'tags': {
+                'some_key1': 'some_value1',
+                'some_key2': 'some_value2'
+            },
+            'url_prefix': '/'
+        }
+        config = self.create_config(sample_app,
+                                    app_name='rest-api-app',
+                                    api_gateway_endpoint_type='REGIONAL',
+                                    api_gateway_custom_domain=domain_name,
+                                    )
+        builder = ApplicationGraphBuilder()
+        application = builder.build(config, stage_name='dev')
+        rest_api = application.resources[0]
+        assert isinstance(rest_api, models.RestAPI)
+        domain_name = rest_api.domain_name
+        api_mapping = domain_name.api_mapping
+        assert isinstance(domain_name, models.DomainName)
+        assert isinstance(api_mapping, models.APIMapping)
+        assert api_mapping.mount_path == '(none)'
 
+    def test_can_build_lambda_function_app_with_vpc_config(
+            self, sample_app_lambda_only
+    ):
         @sample_app_lambda_only.lambda_function()
         def foo(event, context):
             pass
@@ -459,6 +489,31 @@ class TestApplicationGraphBuilder(object):
         assert disconnect_function.resource_name == 'websocket_disconnect'
         assert disconnect_function.handler == 'app.disconnect'
 
+    def test_can_create_websocket_api_with_domain_name(self,
+                                                       sample_websocket_app):
+        domain_name = {
+            'domain_name': 'example.com',
+            'tls_version': 'TLS_1_2',
+            'certificate_arn': 'certificate_arn',
+            'tags': {
+                'tag_key1': 'tag_value1',
+                'tag_key2': 'tag_value2'
+            }
+        }
+        config = self.create_config(sample_websocket_app,
+                                    app_name='websocket-app',
+                                    autogen_policy=True,
+                                    websocket_api_custom_domain=domain_name)
+        builder = ApplicationGraphBuilder()
+        application = builder.build(config, stage_name='dev')
+        websocket_api = application.resources[0]
+        assert isinstance(websocket_api, models.WebsocketAPI)
+
+        domain_name = websocket_api.domain_name
+        assert isinstance(domain_name, models.DomainName)
+        assert isinstance(domain_name.api_mapping, models.APIMapping)
+        assert domain_name.api_mapping.mount_path == '(none)'
+
     def test_can_create_websocket_app_missing_connect(
             self, websocket_app_without_connect):
         config = self.create_config(websocket_app_without_connect,
